﻿// Copyright (C) 2012, Clément Pit--Claudel (http://pit-claudel.fr/clement/blog)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of 
// this software and associated documentation files (the "Software"), to deal in 
// the Software without restriction, including without limitation the rights to 
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
// the Software, and to permit persons to whom the Software is furnished to do so, 
// subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all 
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace StringComparisons {
    class Program {
        static int REPEAT = 20;
        static int COUNT = 100000;
        static int MAX_LENGTH = 500;
        static Random prng = new Random(0);

        static Tuple<double, int, int> compare<T>(List<T> v1, List<T> v2, int iterations) {
            double best = 0;
            int matches = 0, non_matches = 0;

            for (int iteration_id = 0; iteration_id < iterations; iteration_id++) {
                matches = 0; non_matches = 0;

                Stopwatch clock = Stopwatch.StartNew();
                for (int pos = 0; pos < v1.Count; pos++)
                    if (EqualityComparer<T>.Default.Equals(v1[pos], v2[pos]))
                        matches++;
                    else
                        non_matches++;
                clock.Stop();

                double timing = clock.ElapsedTicks / (double)TimeSpan.TicksPerMillisecond;
                best = (iteration_id == 0 ? timing : Math.Min(timing, best));
            }

            return new Tuple<double, int, int>(best, matches, non_matches);
        }

        static List<String> random_strings(int count, int length) {
            string chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
            char[] buffer = new char[length];

            List<String> output = new List<String>(count);
            for (int str_id = 0; str_id < count; str_id++) {
                for (int char_id = 0; char_id < length; char_id++)
                    buffer[char_id] = chars[prng.Next(chars.Length)];

                output.Add(new string(buffer));
            }

            return output;
        }

        static List<int> random_ints(int count) {
            List<int> output = new List<int>(count);
            for (int int_id = 0; int_id < count; int_id++)
                output.Add(prng.Next());

            return output;
        }

        static void Main(string[] args) {
            List<int> int1 = random_ints(COUNT);
            List<int> int2 = random_ints(COUNT);
            List<String> string1 = random_strings(COUNT, MAX_LENGTH);
            List<String> string2 = random_strings(COUNT, MAX_LENGTH);


            Tuple<double, int, int> int_timing = compare<int>(int1, int2, REPEAT);
            Console.WriteLine("C#-Ints	{0}	{1}	{2}	ms ({3} matches, {4} non-matches)", COUNT, "", int_timing.Item1, int_timing.Item2, int_timing.Item3);

            for (int length = 1; length <= MAX_LENGTH; length += 1) {
                List<String> truncated1 = new List<String>(COUNT);
                List<String> truncated2 = new List<String>(COUNT);
                string1.ForEach(str => truncated1.Add(str.Substring(MAX_LENGTH - length)));
                string2.ForEach(str => truncated2.Add(str.Substring(MAX_LENGTH - length)));

                Tuple<double, int, int> str_timing = compare<String>(truncated1, truncated2, REPEAT);
                Console.WriteLine("C#-Strings	{0}	{1}	{2}	ms ({3} matches, {4} non-matches)", COUNT, length, str_timing.Item1, str_timing.Item2, str_timing.Item3);
            }

            Console.ReadLine();
        }
    }
}
